package com.example.PhidgetIRExample;

import android.app.Activity;
import android.content.Context;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.widget.LinearLayout;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.Toast;

import com.phidget22.*;

public class IRExample extends Activity {

	IR ch;
	SeekBar dataIntervalBar;

	Toast errToast;

	int minDataInterval;
	int repeatCount = 0;

	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        //Hide device information and settings until one is attached
		LinearLayout settingsAndData = (LinearLayout) findViewById(R.id.settingsAndData);
		settingsAndData.setVisibility(LinearLayout.GONE);

        try
        {
        	ch = new IR();

        	//Allow direct USB connection of Phidgets
			if(getPackageManager().hasSystemFeature(PackageManager.FEATURE_USB_HOST))
                com.phidget22.usb.Manager.Initialize(this);

			//Enable server discovery to list remote Phidgets
			this.getSystemService(Context.NSD_SERVICE);
			Net.enableServerDiscovery(ServerType.DEVICE_REMOTE);

			ch.addAttachListener(new AttachListener() {
				public void onAttach(final AttachEvent attachEvent) {
				    AttachEventHandler handler = new AttachEventHandler(ch);
                    synchronized(handler)
					{
						runOnUiThread(handler);
						try {
							handler.wait();
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				}
			});

			ch.addDetachListener(new DetachListener() {
				public void onDetach(final DetachEvent detachEvent) {
                    DetachEventHandler handler = new DetachEventHandler(ch);
                    synchronized(handler)
					{
						runOnUiThread(handler);
						try {
							handler.wait();
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				}
			});

			ch.addErrorListener(new ErrorListener() {
				public void onError(final ErrorEvent errorEvent) {
					ErrorEventHandler handler = new ErrorEventHandler(ch, errorEvent);
					runOnUiThread(handler);
				}
			});

			ch.addCodeListener(new IRCodeListener() {
				public void onCode(IRCodeEvent codeEvent) {
                    IRCodeEventHandler handler = new IRCodeEventHandler(ch, codeEvent);
                    runOnUiThread(handler);
                }
			});

			ch.addLearnListener(new IRLearnListener() {
				public void onLearn(IRLearnEvent codeEvent) {
					IRLearnEventHandler handler = new IRLearnEventHandler(ch, codeEvent);
					runOnUiThread(handler);
				}
			});
			
			ch.open();
        } catch (PhidgetException pe) {
	        pe.printStackTrace();
		}

    }

    class AttachEventHandler implements Runnable { 
    	Phidget ch;

		public AttachEventHandler(Phidget ch) {
			this.ch = ch;
		}

		public void run() {
			LinearLayout settingsAndData = (LinearLayout) findViewById(R.id.settingsAndData);
			settingsAndData.setVisibility(LinearLayout.VISIBLE);

			TextView attachedTxt = (TextView) findViewById(R.id.attachedTxt);

			attachedTxt.setText("Attached");
			try {
				TextView nameTxt = (TextView) findViewById(R.id.nameTxt);
				TextView serialTxt = (TextView) findViewById(R.id.serialTxt);
				TextView versionTxt = (TextView) findViewById(R.id.versionTxt);
				TextView channelTxt = (TextView) findViewById(R.id.channelTxt);
				TextView hubPortTxt = (TextView) findViewById(R.id.hubPortTxt);
				TextView labelTxt = (TextView) findViewById(R.id.labelTxt);

				nameTxt.setText(ch.getDeviceName());
				serialTxt.setText(Integer.toString(ch.getDeviceSerialNumber()));
				versionTxt.setText(Integer.toString(ch.getDeviceVersion()));
				channelTxt.setText(Integer.toString(ch.getChannel()));
				hubPortTxt.setText(Integer.toString(ch.getHubPort()));
				labelTxt.setText(ch.getDeviceLabel());

			} catch (PhidgetException e) {
				e.printStackTrace();
			}

			//notify that we're done
			synchronized(this)
			{
				this.notify();
			}
		}
    }
    
    class DetachEventHandler implements Runnable {
    	Phidget ch;
    	
    	public DetachEventHandler(Phidget ch) {
    		this.ch = ch;
    	}
    	
		public void run() {
			LinearLayout settingsAndData = (LinearLayout) findViewById(R.id.settingsAndData);

			settingsAndData.setVisibility(LinearLayout.GONE);

			TextView attachedTxt = (TextView) findViewById(R.id.attachedTxt);
			attachedTxt.setText("Detached");

			TextView nameTxt = (TextView) findViewById(R.id.nameTxt);
			TextView serialTxt = (TextView) findViewById(R.id.serialTxt);
			TextView versionTxt = (TextView) findViewById(R.id.versionTxt);
			TextView channelTxt = (TextView) findViewById(R.id.channelTxt);
			TextView hubPortTxt = (TextView) findViewById(R.id.hubPortTxt);
			TextView labelTxt = (TextView) findViewById(R.id.labelTxt);

			nameTxt.setText(R.string.unknown_val);
			serialTxt.setText(R.string.unknown_val);
			versionTxt.setText(R.string.unknown_val);
			channelTxt.setText(R.string.unknown_val);
			hubPortTxt.setText(R.string.unknown_val);
			labelTxt.setText(R.string.unknown_val);


			((TextView)findViewById(R.id.codeTxt)).setText("");
			((TextView)findViewById(R.id.bitCountTxt)).setText("");
			((TextView)findViewById(R.id.repeatCountTxt)).setText("");

			((TextView)findViewById(R.id.learnedCodeTxt)).setText("");
			((TextView)findViewById(R.id.bitLengthTxt)).setText("");
			((TextView)findViewById(R.id.encodingTxt)).setText("");
			((TextView)findViewById(R.id.gapTxt)).setText("");
			((TextView)findViewById(R.id.trailTxt)).setText("");
			((TextView)findViewById(R.id.zeroTxt)).setText("");
			((TextView)findViewById(R.id.oneTxt)).setText("");
			((TextView)findViewById(R.id.headerTxt)).setText("");
			((TextView)findViewById(R.id.repeatTxt)).setText("");
			((TextView)findViewById(R.id.toggleMaskTxt)).setText("");
			
			//notify that we're done
			synchronized(this)
			{
				this.notify();
			}
		}
    }

	class ErrorEventHandler implements Runnable {
		Phidget ch;
		ErrorEvent errorEvent;

		public ErrorEventHandler(Phidget ch, ErrorEvent errorEvent) {
			this.ch = ch;
			this.errorEvent = errorEvent;
		}

		public void run() {
			 if (errToast == null)
				 errToast = Toast.makeText(getApplicationContext(), errorEvent.getDescription(), Toast.LENGTH_SHORT);

			 //replace the previous toast message if a new error occurs
			 errToast.setText(errorEvent.getDescription());
			 errToast.show();
        }
	}

	class IRCodeEventHandler implements Runnable {
		Phidget ch;
		IRCodeEvent codeEvent;

		public IRCodeEventHandler(Phidget ch, IRCodeEvent codeEvent) {
			this.ch = ch;
			this.codeEvent = codeEvent;
		}

		public void run() {
		    TextView codeTxt = (TextView)findViewById(R.id.codeTxt);
			TextView repeatCountTxt = (TextView)findViewById(R.id.repeatCountTxt);
			TextView bitCountTxt = (TextView)findViewById(R.id.bitCountTxt);

			if(codeEvent.getIsRepeat())
				repeatCount++;
			else
				repeatCount = 0;

			codeTxt.setText("0x" + codeEvent.getCode());
			repeatCountTxt.setText(String.valueOf(repeatCount));
			bitCountTxt.setText(String.valueOf(codeEvent.getBitCount()));
		}
	}

	class IRLearnEventHandler implements Runnable {
		Phidget ch;
		IRLearnEvent learnEvent;

		public IRLearnEventHandler(Phidget ch, IRLearnEvent learnEvent) {
			this.ch = ch;
			this.learnEvent = learnEvent;
		}

		public void run() {
			((TextView)findViewById(R.id.learnedCodeTxt)).setText("0x" + learnEvent.getCode());
			((TextView)findViewById(R.id.bitLengthTxt)).setText(String.valueOf(learnEvent.getCodeInfo().bitCount));
			((TextView)findViewById(R.id.encodingTxt)).setText(learnEvent.getCodeInfo().encoding.getMessage());
			((TextView)findViewById(R.id.gapTxt)).setText(String.valueOf(learnEvent.getCodeInfo().gap));
			((TextView)findViewById(R.id.trailTxt)).setText(String.valueOf(learnEvent.getCodeInfo().trail));
			((TextView)findViewById(R.id.zeroTxt)).setText(
					String.valueOf(learnEvent.getCodeInfo().zero[0]) + ", "
							+ String.valueOf(String.valueOf(learnEvent.getCodeInfo().zero[1]))
			);
			((TextView)findViewById(R.id.oneTxt)).setText(
					String.valueOf(learnEvent.getCodeInfo().one[0]) + ", "
							+ String.valueOf(String.valueOf(learnEvent.getCodeInfo().one[1]))
			);
			((TextView)findViewById(R.id.headerTxt)).setText(
					String.valueOf(learnEvent.getCodeInfo().header[0]) + ", "
							+ String.valueOf(String.valueOf(learnEvent.getCodeInfo().header[1]))
			);

			int[] repeat = learnEvent.getCodeInfo().repeat;
			String repeatTxt = "";
			for(int i = 0; i < repeat.length; i++) {
				if(repeat[i] == 0)
					break;

				repeatTxt += String.valueOf(repeat[i]);
				repeatTxt += ", ";
			}

			((TextView)findViewById(R.id.repeatTxt)).setText(repeatTxt);

			if(!learnEvent.getCodeInfo().toggleMask.isEmpty())
				((TextView)findViewById(R.id.toggleMaskTxt)).setText("0x" + learnEvent.getCodeInfo().toggleMask);
			else
				((TextView)findViewById(R.id.toggleMaskTxt)).setText("None");
		}
	}

    @Override
    protected void onDestroy() {
    	super.onDestroy();
    	try {
			ch.close();

		} catch (PhidgetException e) {
			e.printStackTrace();
		}

		//Disable USB connection to Phidgets
    	if(getPackageManager().hasSystemFeature(PackageManager.FEATURE_USB_HOST))
            com.phidget22.usb.Manager.Uninitialize();
    }

}

